﻿using System;
using System.ComponentModel;
using System.Net;
using System.Threading;
using System.Threading.Tasks;

namespace AsyncCSharp
{
    internal class Program
    {
        static string _workStatus = "Forward warp speed!";

        static void displayStatus()
        {
            lock (_workStatus)
            Console.WriteLine("C# status: " + _workStatus);
        }

        static void updateStatus(string newStatus)
        {
            lock (_workStatus)
            _workStatus = newStatus;
        }

        static string getNewStatus(string name)
        {
            //throw new InvalidOperationException("Yikes!");
            Thread.Sleep(3000);
            return "He's dead, " + name + "!";
        }

        static void updateStatusSync(string name)
        {
            var newStatus = getNewStatus(name);
            updateStatus(newStatus);
        }

        static void updateStatusNewThread(string name)
        {
            var workerThread = new Thread(() => updateStatusSync(name));
            workerThread.Start();
        }

        static void updateStatusThreadPool(string name)
        {
            ThreadPool.QueueUserWorkItem(
                new WaitCallback(obj => updateStatusSync(name)));
        }

        static void updateStatusAsyncDelegate(string name)
        {
            Func<string> getStatus =
                () => getNewStatus(name);

            getStatus.BeginInvoke
                (
                    arg =>
                        {
                            var newStatus = getStatus.EndInvoke(arg);
                            updateStatus(newStatus);
                        }
                    , null);
        }

        static void updateStatusAsyncEvent(string name)
        {
            var backgroundWorker = new BackgroundWorker();

            backgroundWorker.DoWork +=
                (sender, doWorkArgs) =>
                {
                    doWorkArgs.Result = getNewStatus(name);
                };

            backgroundWorker.RunWorkerCompleted +=
                (sender, workerCompletedArgs) =>
                {
                    var newStatus = workerCompletedArgs.Result as string;
                    updateStatus(newStatus);
                };

            backgroundWorker.RunWorkerAsync();
        }

        static void updateStatusAsyncTask_4_0(string name)
        {
            Task<String> task = Task.Run(() => getNewStatus(name));
            task.ContinueWith(new Action<Task<string>>(
                (newTask) => 
                    updateStatus(newTask.Result)));
        }

        static async void updateStatusAsyncTask(string name)
        {
            Task<string> task = Task.Run(() => getNewStatus(name));
            string newStatus = await task;
            updateStatus(newStatus);
        }

        static async void updateStatusAsyncTask2(string name)
        {
            await Task.Run(() => updateStatusSync(name));
        }

        static void Main(string[] args)
        {
            displayStatus();

            //updateStatusSync("Jim");
            //updateStatusNewThread("Uhuru");
            //updateStatusThreadPool("Chekhov");
            //updateStatusAsyncDelegate("Sulu");
            //updateStatusAsyncEvent("Spock");
            //updateStatusAsyncTask_4_0("Khan");
            //updateStatusAsyncTask("Scotty");
            updateStatusAsyncTask2("Bones");

            while (_workStatus == "Forward warp speed!")
            {
                displayStatus();
                Thread.Sleep(500);
            }

            displayStatus();
            Console.ReadKey();
        }

        // from Petracik
        static async Task<Tuple<string, int>> DownloadPage(string url)
        {
            var request = WebRequest.Create(url);
            using (var response = await request.GetResponseAsync())
            {
                using (var stream = response.GetResponseStream())
                {
                    return Tuple.Create("Fububu", 1);
                }
            }
        }
    }
}