﻿open System
open System.Threading
open System.ComponentModel
open System.Threading.Tasks

let  _workStatus = ref "Forward warp speed!"

let displayStatus() =
    lock _workStatus (fun () -> Console.WriteLine("F# status: " + !_workStatus))

let getNewStatus name =
    Thread.Sleep(2000)
    "He's dead, " + name + "!"

let updateStatus newStatus =
    lock _workStatus (fun () -> _workStatus := newStatus)

let updateStatusSync name = 
    getNewStatus name |> updateStatus 

let updateStatusNewThread name =
    let workerThread = new Thread(fun () -> updateStatusSync(name))
    workerThread.Start()

let updateStatusThreadPool name =
    ThreadPool.QueueUserWorkItem(
                new WaitCallback(fun obj -> updateStatusSync(name)))

let updateStatusAsyncDelegate name =
    let getStatus = new Func<_>(fun () ->
        getNewStatus name)
    getStatus.BeginInvoke
        (new AsyncCallback(
            fun arg -> getStatus.EndInvoke(arg) |> updateStatus), null)

let updateStatusAsyncEvent name =
    let backroundWorker = new BackgroundWorker()
    backroundWorker.DoWork.Add( 
        (fun doWorkArgs  ->
            doWorkArgs.Result <- getNewStatus name))
    backroundWorker.RunWorkerCompleted.Add(
        (fun workerCompletedArgs ->
            workerCompletedArgs.Result :?> string |> updateStatus))
    backroundWorker.RunWorkerAsync()

let updateStatusAsyncWorkflowDelegate name =
    
    let getStatus = new Func<_>(fun _ ->
        getNewStatus name)
    async
        {
            let! newStatus = 
                Async.FromBeginEnd(
                    getStatus.BeginInvoke, 
                    getStatus.EndInvoke)

            do updateStatus newStatus
        } 
    |> Async.Start

let updateStatusAsyncTask_4_0 name =

    let getStatus = new Func<_>(fun () ->
        getNewStatus name)

    Task.Run(getStatus).ContinueWith(
        new Action<Task<string>>(fun task ->
            updateStatus task.Result))

let updateStatusAsyncWorkflowTask name =

    let getStatus = new Func<_>(fun () ->
        getNewStatus name)

    async
        {
            let! newStatus =
                Async.AwaitTask( 
                    Task<string>.Factory.StartNew(getStatus))
            do updateStatus newStatus    
        }
    |> Async.Start

let updateStatusAsyncWorkflowTask2 name =

    let getStatus = new Func<_>(fun () ->
        updateStatusSync name)

    async
        {
            do! Async.AwaitTask(Task.Run(getStatus))
        }
    |> Async.Start

[<EntryPoint>]
let main argv = 
    
    displayStatus() 
    
    //updateStatusSync "Jim"
    //updateStatusNewThread "Uhuru"
    //updateStatusThreadPool("Chekhov")
    //updateStatusAsyncEvent "Spock"
    //updateStatusAsyncDelegate "Sulu"
    //updateStatusAsyncTask_4_0 "Khan"
    //updateStatusAsyncWorkflowDelegate "Tribble"
    //updateStatusAsyncWorkflowTask "Scotty"
    updateStatusAsyncWorkflowTask2 "Bones"
    |> ignore

    while !_workStatus = "Forward warp speed!" do
        displayStatus()
        Thread.Sleep(500)

    displayStatus() 

    Console.ReadKey() |> ignore

    0 // return an integer exit code
