﻿Imports System.Threading
Imports System.ComponentModel
Imports System.Threading.Tasks

Module Module1

    Dim _workStatus As String = "Forward warp speed!"

    Sub displayStatus()
        SyncLock (_workStatus)
            Console.WriteLine("VB status: " + _workStatus)
        End SyncLock
    End Sub

    Sub updateStatus(newStatus As String)
        SyncLock (_workStatus)
            _workStatus = newStatus
        End SyncLock
    End Sub

    Function getNewStatus(name As String) As String
        Thread.Sleep(2000)
        Return "He's dead, " + name + "!"
    End Function

    Sub updateStatusSync(name As String)
        Dim newStatus = getNewStatus(name)
        updateStatus(newStatus)
    End Sub

    Sub updateStatusNewThread(name As String)
        Dim workerThread = New Thread(Sub() updateStatusSync(name))
        workerThread.Start()
    End Sub

    Sub updateStatusThreadPool(name As String)
        ThreadPool.QueueUserWorkItem(
            New WaitCallback(Sub(obj) updateStatusSync(name)))
    End Sub

    Sub updateStatusAsyncDelegate(name As String)

        Dim getStatus As Func(Of String) =
            Function() getNewStatus(name)

        getStatus.BeginInvoke(Sub(arg) updateStatus(getStatus.EndInvoke(arg)), Nothing)

    End Sub

    Sub updateStatusAsyncEvent(name As String)

        Dim backroundWorker = New BackgroundWorker()
        AddHandler backroundWorker.DoWork, Sub(sender, doWorkArgs)
                                               doWorkArgs.Result = getNewStatus(name)
                                           End Sub
        AddHandler backroundWorker.RunWorkerCompleted, Sub(sender, workerCompletedArgs)
                                                           updateStatus(workerCompletedArgs.Result)
                                                       End Sub
        backroundWorker.RunWorkerAsync()
    End Sub

    Sub updateStatusAsyncTask_4_0(name As String)
        Dim task = System.Threading.Tasks.Task.Run(Function() getNewStatus(name))
        task.ContinueWith(Sub(newTask) updateStatus(newTask.Result))
    End Sub

    Async Sub updateStatusAsyncTask(name As String)
        Dim getStatus = Task(Of String).Factory.StartNew(Function() getNewStatus(name))
        Dim newStatus = Await getStatus
        updateStatus(newStatus)
    End Sub

    Async Sub updateStatusAsyncTask2(name As String)
        Await Task.Run(Sub() updateStatusSync(name))
    End Sub

    Sub Main()

        displayStatus()

        'updateStatusSync("Jim")
        'updateStatusNewThread("Uhuru")
        'updateStatusThreadPool("Chekhov")
        'updateStatusAsyncDelegate("Sulu")
        'updateStatusAsyncEvent("Spock");
        updateStatusAsyncTask_4_0("Khan")
        'updateStatusAsyncTask("Scotty")
        'updateStatusAsyncTask2("Bones")

        Do While _workStatus = "Forward warp speed!"
            displayStatus()
            Thread.Sleep(500)
        Loop

        displayStatus()
        Console.ReadKey()

    End Sub

End Module
