﻿using System;
using System.Collections.Generic;
using System.Threading;
using System.Threading.Tasks;

namespace MultiAsyncCSharp
{
    class Program
    {
        static async Task cancelOneTask(string message, int timeout, CancellationTokenSource cancelationSource)
        {
            await Task.Factory.StartNew(
                () =>
                    {
                        while (!cancelationSource.IsCancellationRequested)
                        {
                            Console.WriteLine(message);
                            Thread.Sleep(timeout);
                            if (message.Contains("He's dead"))
                            {
                                throw new InvalidOperationException(message);
                            }
                        }
                    }
                , cancelationSource.Token);

            Console.WriteLine("Canceled!");
        }

        static async void cancelManyTasks(CancellationTokenSource cancelationSource)
        {
            var tasks = new List<Task>
                {
                    cancelOneTask("Hello my baby", 700, cancelationSource),
                    cancelOneTask("Hello my honey", 900, cancelationSource),
                    cancelOneTask("Hello my ragtime gal", 1100, cancelationSource)
                };
            await Task.WhenAll(tasks);
        }

        static async void handleManyExceptions(CancellationTokenSource cancelationSource)
        {
            var tasks = new List<Task>
                {
                    cancelOneTask("He's dead, Jim", 5000, cancelationSource),
                    cancelOneTask("He's alive, Spock", 5000, cancelationSource),
                    cancelOneTask("He's dead, Scotty", 5000, cancelationSource)
                };
            try
            {
                await Task.WhenAll(tasks);
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
            }
        }

        static void Main(string[] args)
        {

            var cancelationSource = new CancellationTokenSource();

            Console.WriteLine("Press any key to cancel");

            cancelOneTask("Waiting for cancelation", 1000, cancelationSource);
            //cancelManyTasks(cancelationSource);
            //handleManyExceptions(cancelationSource);

            Console.ReadKey();
            cancelationSource.Cancel();

            Console.WriteLine();
            Console.WriteLine("Finished!");
            Console.ReadKey();
        }
    }
}
