﻿using System;
using System.Linq;
using System.Net;
using System.Threading.Tasks;

namespace ParallelCSharp
{
    class Program
    {
        static Tuple<string, string>[] _webSites =
            {
                Tuple.Create("Google", "http://google.com"),
                Tuple.Create("Microsoft", "http://microsoft.com"),
                Tuple.Create("Bennett Adelson", "http://www.bennettadelson.com/"),
                Tuple.Create("Amazon", "http://www.amazon.com/"),
                Tuple.Create("Facebook", "http://www.facebook.com/")
            };

        static void getWebContentLengths()
        {
            Parallel.For(0, _webSites.Length,
                         index =>
                             {
                                 var siteInfo = _webSites[index];
                                 var webClient = new WebClient();
                                 var htmlContent = webClient.DownloadString(siteInfo.Item2);
                                 Console.WriteLine("{0} has {1} characters", siteInfo.Item1,
                                                   htmlContent.Length);
                             });
        }

        private static void sumWebContentLengths()
        {
            var sumLengths =
                _webSites
                    .AsParallel()
                    .Select(siteInfo =>
                        {
                            var webClient = new WebClient();
                            var htmlContent = webClient.DownloadString(siteInfo.Item2);
                            return htmlContent.Length;
                        }
                    )
                    .Sum();
            Console.WriteLine("The sum of the site lengths is {0}", sumLengths);
        }

        static void Main(string[] args)
        {
            Console.WriteLine("ParallelCSharp");
            getWebContentLengths();
            Console.WriteLine();
            sumWebContentLengths();

            Console.WriteLine("Press any key to exit ...");
            Console.ReadKey();
        }
    }
}
